;==============================================================================
; Video RAM Address
;==============================================================================
; Description:
;   Entering a pair of screen coordinates as parameters, these subroutines 
;   return the corresponding video RAM address.
; Author:
;   TK90X Fan 
;   (reikainosuke-tk@yahoo.com.br, http://cantinhotk90x.blogspot.com.br)
; Version:
;   1.0 (Jan/24/2015)
;==============================================================================
; Assembler labels:
;   VideoPixAddChk
;   VideoPixelAdd
;   VideoByteAdd
;==============================================================================

;------------------------------------------------------------------------------
; Video Pixel Address with parameter Checking (VideoPixAddChk)
; This subroutine accepts as parameters, a pair of coordinates defining a pixel 
; position, and it returns the corresponding address at video RAM and 1-bit 
; mask with the position of pixel within the byte. The vertical coordinate is
; checked and carry flag will be raised if illegal value is found.
;------------------------------------------------------------------------------
; Parameters:
;       B=vertical coordinate  (0-191)
;       C=horizontal coordinate (0-255)
; Output:
;       HL=address at video RAM (unchanged if B>=192)
;       E=1-bit mask of pixel for byte pointed by HL (E=0 if B>=192)
;       B is unchanged (vertical coordinate)
;       C=byte column position (0-31) (unchanged if B>=192)
;       A=H (191 if B>=192)
;       Flags: C=0 if B<192; Z=0; S=0
;------------------------------------------------------------------------------
; This routine check if vertical coordinate value is inside screen boundary
; (0-192) before executing 'VideoPixelAdd'. Checking procedure imposes overhead
; that can be avoided running VideoPixelAdd directly, but wrong value can cause
; nasty bug.
; The error condition can be monitored by carry flag but 
;------------------------------------------------------------------------------

VideoPixAddChk
    LD A,191                ; The carry flag is raised if 191<B 
    CP B
    LD E,0                  ; E value is 0
    RET C                   ; Return if error condition is detected

; Continue through the VideoPixelAdd subroutine

;------------------------------------------------------------------------------
; Video Pixel Address (VideoPixelAdd)
; This subroutine accepts as parameters, a pair of coordinates defining a pixel 
; position, and it returns the corresponding address at video RAM and 1-bit 
; mask with the position of pixel within the byte.
;------------------------------------------------------------------------------
; Parameters:
;       B=vertical coordinate  (0-191)
;       C=horizontal coordinate (0-255)
; Output:
;       HL=address at video RAM
;       E=1-bit mask of pixel for byte pointed by HL
;       B is unchanged (vertical coordinate)
;       C=byte column position (0-31)
;       A=H
;       Flags: C=0; Z=0; S=0
;------------------------------------------------------------------------------
; The video RAM address is returned in HL register pair and 1-bit mask is
; returned in E register. The content of (HL) would be modified using E:
;   LD A,E : OR (HL) : LD (HL),A            to plot pixel
;   LD A,E : CPL : AND (HL) : LD (HL),A     to erase pixel
;   LD A,E : XOR (HL) : LD (HL),A           to invert pixel
;
; In this routine, there is an assumption that a screen  position is specified
; by a pair of coordinates. The horizontal coordinate rises from the left (0) to 
; the right (255) of screen. The vertical coordinate rises from the top (0) to 
; the bottom (191) of screen. No attempt is made to check if the coordinate
; values are within the allowed range. Thus, wrong value will result in 
; unpredictable errors.
;
; This routine has bigger size because it is optimized for speed.
;------------------------------------------------------------------------------

VideoPixelAdd
    LD A,128                    ; Set bit 7 to position pixel mask to left edge
    SRL C                       ; C=%0cccccpp
    JP NC,VideoPixelAdd.j0      ; If p0 is 1, shift mask 1 position to right
    RRCA
VideoPixelAdd.j0
    SRL C                       ; C=%00cccccp
    JP NC,VideoPixelAdd.j1      ; If p1 is 1, shift mask 2 positions to right
    RRCA
    RRCA
VideoPixelAdd.j1
    SRL C                       ; C=%000ccccc
    JP NC,VideoPixelAdd.j2      ; If p1 is 1, shift mask 3 positions to right
    RRCA
    RRCA
    RRCA
    RRCA
VideoPixelAdd.j2
    LD E,A                      ; Store pixel mask into E register

; Continue through the VideoByteAdd subroutine

;------------------------------------------------------------------------------
; Video Byte Address (VideoByteAdd)
; Return video RAM address of the byte at a given screen position
;------------------------------------------------------------------------------
; Parameters:
;       B=vertical coordinate  (0-191)
;       C=horizontal coordinate (0-31)
; Output:
;       HL=address at video RAM
;       BC is unchanged (coordinates)
;       A=H
;       Flags: C=0; Z=0; S=0
;------------------------------------------------------------------------------
; ZX Spectrum stores bitmap of video at RAM, starting from 16384 ($4000). Each
; byte holds on/off state of 8 contiguous pixels, corresponding to the width of
; one character. 32 contiguous bytes form 1-pixel high line which width is equal
; to 32 characters or 256 pixels.
;
; The address of line is not simply. One character line is composed by 8 pixel
; lines and the entire screen is composed by three areas, each one with 8
; character lines. This scheme is not intuitive and conversion from coordenates
; to memory address requires a lot of bitwise operations.
;
; In this routine, there is an assumption that a screen  position is specified
; by a pair of coordinates. The horizontal coordinate rises from the left (0) to 
; the right (31) of screen. The vertical coordinate rises from the top (0) to 
; the bottom (191) of screen. No attempt is made to check if the coordinate
; values are within the allowed range. Thus, wrong values will result in 
; unpredictable errors.
;------------------------------------------------------------------------------

VideoByteAdd
    LD A,B                      ; A=%aalllrrr
    RLCA                        ; A=%alllrrra
    RLCA                        ; A=%lllrrraa
    AND %11100000               ; A=%lll00000
    OR C                        ; A=%lllccccc
    LD L,A                      ; LSB of address is done
    LD A,B                      ; A=%aalllrrr
    RRCA                        ; A=%raalllrr
    RRCA                        ; A=%rraalllr
    RRCA                        ; A=%rrraalll
    AND %00011000               ; A=%000aa000
    LD H,A
    LD A,B                      ; A=%aalllrrr
    AND %00000111               ; A=%00000rrr
    OR H                        ; A=%000aarrr
    OR %01000000                ; A=%010aarrr
    LD H,A                      ; MSB of address is done
    RET                         ; End of subroutine

